--[[ 
v1.07	6/13/13		Added compatibility with ShammyShields v3.1.

v1.06	5/12/2013	Added compatibility with the Healbot v5.2.05 new multi-frame structure.
		Note:
			- v5.2.0.5 using multi-frames "f1_HealBot_Action" thru "f10_HealBot_Action"
			- v4.3.0.0 thru v5.2.0.4 using main frame "HealBot_Action"

v1.05	9/15/2012	Fixed spamming issue when reload the UI while in combat.

v1.04	9/1/2012	Updated the TOC for v5.04

v1.03	8/6/2012	Fixed spamming issue while in combat.

v1.02	6/23/2012	Minor typographical correction.  No change in function.

v1.01	5/26/2012	Added check for being in combat.  Changing anchoring while in combat cannot be done and would fail.

v1.0	2/20/2012	Initial public release
 ]]
--local vars
local LastUpdate = 0
local UpdateInterval = 1
local VarsLoaded = false
local InCombatNotice = false
local ShammyShieldsVer = tonumber(GetAddOnMetadata("ShammyShields", "Version"))

--Global vars
KSSAnchor_ShammyShields_ESIcon = false
KSSAnchor_Method = "off"
KSSAnchor_AttachFrame = ""

local function print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("KSSAnchor: " .. tostring(msg))
end

local function Help()
	print("Command = /KSSA or /SSAnchor.")
	print("Usage = Command [Position] [HBFrameName].")
	print("Positions = [left||right||top||bottom||off].")
	print("Healbot Frame = [self||main tanks||healers||group||my targets||focus||raid||pets||vehicle||targets].")
	print("Example: /kssa top main tanks.")
end

function KSSAnchor_OnLoad(self)
	SlashCmdList["KSSAnchor"]=KSSAnchor_Command
	SLASH_KSSAnchor1,SLASH_KSSAnchor2 = "/KSSA","/SSAnchor"
    KSSAnchorFrame:RegisterEvent("ADDON_LOADED")
	KSSAnchorFrame:RegisterEvent("PLAYER_LOGIN")
	print("Loaded.")
	Help()
end

local function StrSplit(Text, Seperator)
	local Chunk
	if Text ~= nil and Text ~= "" then
		if Seperator == nil then
			Seperator = "%s"
		end
		Ret={} ; i=1
		for Chunk in string.gmatch(Text, "([^"..Seperator.."]+)") do
			Ret[i] = Chunk
			i = i + 1
		end
		return Ret
	else
		return Text
	end
end

function GetHBFrameInfo(Frame)
	local FrameTable = nil
	local FrameName = nil
	local FrameNumber = nil
	local FrameVisible = nil
	local i = nil
	local ii = nil
	
	local HBFrames = {
		{["Table"] = f1_HealBot_Action, ["Visible"] = f1_HealBot_Action:IsVisible()},
		{["Table"] = f2_HealBot_Action, ["Visible"] = f2_HealBot_Action:IsVisible()},
		{["Table"] = f3_HealBot_Action, ["Visible"] = f3_HealBot_Action:IsVisible()},
		{["Table"] = f4_HealBot_Action, ["Visible"] = f4_HealBot_Action:IsVisible()},
		{["Table"] = f5_HealBot_Action, ["Visible"] = f5_HealBot_Action:IsVisible()},
		{["Table"] = f6_HealBot_Action, ["Visible"] = f6_HealBot_Action:IsVisible()},
		{["Table"] = f7_HealBot_Action, ["Visible"] = f7_HealBot_Action:IsVisible()},
		{["Table"] = f8_HealBot_Action, ["Visible"] = f8_HealBot_Action:IsVisible()},
		{["Table"] = f9_HealBot_Action, ["Visible"] = f9_HealBot_Action:IsVisible()},
		{["Table"] = f10_HealBot_Action, ["Visible"] = f10_HealBot_Action:IsVisible()}
	}
	local HBButtons = {
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil},
		{["Table"] = nil, ["Name"] = nil}
	}
	if HealBot_Action_Header1Bar_text then 
		HBButtons[1].Table = HealBot_Action_Header1Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[1].Name = HealBot_Action_Header1Bar_text:GetText(); 
	end
	if HealBot_Action_Header2Bar_text then 
		HBButtons[2].Table = HealBot_Action_Header2Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[2].Name = HealBot_Action_Header2Bar_text:GetText(); 
	end
	if HealBot_Action_Header3Bar_text then 
		HBButtons[3].Table = HealBot_Action_Header3Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[3].Name = HealBot_Action_Header3Bar_text:GetText(); 
	end
	if HealBot_Action_Header4Bar_text then 
		HBButtons[4].Table = HealBot_Action_Header4Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[4].Name = HealBot_Action_Header4Bar_text:GetText(); 
	end
	if HealBot_Action_Header5Bar_text then 
		HBButtons[5].Table = HealBot_Action_Header5Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[5].Name = HealBot_Action_Header5Bar_text:GetText(); 
	end
	if HealBot_Action_Header6Bar_text then 
		HBButtons[6].Table = HealBot_Action_Header6Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[6].Name = HealBot_Action_Header6Bar_text:GetText(); 
	end
	if HealBot_Action_Header7Bar_text then 
		HBButtons[7].Table = HealBot_Action_Header7Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[7].Name = HealBot_Action_Header7Bar_text:GetText(); 
	end
	if HealBot_Action_Header8Bar_text then 
		HBButtons[8].Table = HealBot_Action_Header8Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[8].Name = HealBot_Action_Header8Bar_text:GetText(); 
	end
	if HealBot_Action_Header9Bar_text then 
		HBButtons[9].Table = HealBot_Action_Header9Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[9].Name = HealBot_Action_Header9Bar_text:GetText(); 
	end
	if HealBot_Action_Header10Bar_text then 
		HBButtons[10].Table = HealBot_Action_Header10Bar_text:GetParent():GetParent():GetParent(); 
		HBButtons[10].Name = HealBot_Action_Header10Bar_text:GetText(); 
	end

	if type(Frame) == "string" then
		FrameName = Frame
		for i = 1, 10, 1 do
			if HBButtons[i].Name then
				if strlower(FrameName) == strlower(HBButtons[i].Name) then
					FrameName = HBButtons[i].Name
					FrameTable = HBButtons[i].Table
					for ii = 1, 10, 1 do
						if HBButtons[i].Table == HBFrames[ii].Table then
							FrameNumber = ii							
							FrameVisible = HBFrames[ii].Visible
						end
					end
				end
			end
		end
	elseif type(Frame) == "number" then
		FrameNumber = Frame
		FrameTable = HBFrames[Frame].Table
		FrameVisible = HBFrames[Frame].Visible
		for i = 1, 10, 1 do
			if HBButtons[i].Table == HBFrames[Frame].Table then
				FrameName = HBButtons[i].Name
			end
		end
	elseif type(Frame) == "table" then
		FrameTable = Frame
		for i = 1, 10, 1 do
			if FrameTable == HBFrames[i].Table then
				FrameNumber = i
				FrameVisible = HBFrames[i].Visible
				for ii = 1, 10, 1 do
					if HBButtons[ii].Table == HBFrames[i].Table then
						FrameName = HBButtons[ii].Name
					end
				end
			end
		end
	end

	return FrameTable, FrameName, FrameNumber, FrameVisible
end

function KSSAnchor_Command(Msg,Editbox)

	local i
	local FrameTable, FrameName, FrameNumber, FrameVisible
	local TargetFrameAnchor = nil
	local Offset = 0
	
	Msg=strlower(Msg)
	Commands = StrSplit(Msg)
 	if #Commands == 0 or Commands[1] ~= "left" and Commands[1] ~= "right" and Commands[1] ~= "top" and Commands[1] ~= "bottom" and Commands[1] ~= "off"  then
		Help()
	else
		if IsAddOnLoaded("HealBot") and IsAddOnLoaded("ShammyShields") then
			if not InCombatLockdown() then
				InCombatNotice = false
				if Commands[1]=="left" or Commands[1]=="right" or Commands[1]=="top" or Commands[1]=="bottom" and #Commands == 2 then
					--find frame user requested by the frame bar text i.e. "Self"
					if #Commands == 3 then
						FrameTable, FrameName, FrameNumber, FrameVisible = GetHBFrameInfo(Commands[2].." "..Commands[3])
					else
						FrameTable, FrameName, FrameNumber, FrameVisible = GetHBFrameInfo(Commands[2])
					end
					if (FrameTable ~= nil) and (FrameVisible ~= nil) then
						-- found matching frame
						if strlower(Commands[1]) == "top" then
							if ShammyShieldsVer < 3.1 then
								Offset = ShammyShields_BG:GetHeight() - 85
							else
								Offset = 0
							end
							TargetFrameAnchor = "bottom"
						end
						if strlower(Commands[1]) == "bottom" then TargetFrameAnchor = "top"; end
						if strlower(Commands[1]) == "left" then TargetFrameAnchor = "right"; end
						if strlower(Commands[1]) == "right" then TargetFrameAnchor = "left"; end
						KSSAnchor_Method = Commands[1]
						if #Commands == 3 then
							KSSAnchor_AttachFrame = Commands[2].." "..Commands[3]
						else
							KSSAnchor_AttachFrame = Commands[2]
						end
						print("Anchoring on "..KSSAnchor_Method.." of "..KSSAnchor_AttachFrame)
						ShammyShields:ClearAllPoints()
						ShammyShields:SetPoint(TargetFrameAnchor,FrameTable,Commands[1],0,Offset)
					elseif FrameTable == nil then
						print("HB Frame does not exist.")
					elseif FrameVisible == nil then
						print("HB Frame is not visible.")
					end
				elseif Commands[1]=="off" then
					print("Anchoring turned off.")
					KSSAnchor_Method = Commands[1]
					ShammyShields:ClearAllPoints()
				elseif Commands[1]=="disabled" then
					print("Anchoring disabled.")
					ShammyShields:ClearAllPoints()
				else
					Help()
				end
			elseif InCombatLockdown() and not InCombatNotice then
				InCombatNotice = true
				print("Cannot set while in combat.")
			end
		else
			if not IsAddOnLoaded("HealBot") and not IsAddOnLoaded("ShammyShields") then
				print("Both HealBot and ShammyShields are not loaded.")
			elseif IsAddOnLoaded("HealBot") and not IsAddOnLoaded("ShammyShields") then
				print("ShammyShields is not loaded.")
			elseif not IsAddOnLoaded("HealBot") and IsAddOnLoaded("ShammyShields") then
				print("HealBot is not loaded.")
			end
		end
	end
end

function KSSAnchor_OnUpdate(self,elapsed)
	
	if VarsLoaded == true then
		LastUpdate = LastUpdate + elapsed
		
		--look for frame size changes due to earth shield and re-Anchor if needed
		if (LastUpdate > UpdateInterval) then
			if IsAddOnLoaded("HealBot") and IsAddOnLoaded("ShammyShields") and not InCombatLockdown() then --check for the frames
				local _, relativeTo, _, _, _ = ShammyShields:GetPoint(1) --get the current anchor frame
				local FrameTable, FrameName, FrameNumber, FrameVisible = GetHBFrameInfo(KSSAnchor_AttachFrame)
				if KSSAnchor_Method ~= "off" and FrameVisible then
					--healbot is visable and we should be anchorod to it so check everything
					if relativeTo ~= FrameTable then --is it anchored to HealBot?
						--HealBot_Action is not currently ShammyShields Parent to anchor it,
						--i.e. not Anchored but should be so lets reanchor it.
						if KSSAnchor_Method ~= "off" then --but not if its set to off
							KSSAnchor_Command(KSSAnchor_Method.." "..KSSAnchor_AttachFrame)
						end
					elseif KSSAnchor_ShammyShields_ESIcon ~= ShammyShields_ESIcon:IsVisible() then --has EarthShield status changed?
						KSSAnchor_ShammyShields_ESIcon = ShammyShields_ESIcon:IsVisible() --save new status
						if KSSAnchor_Method == "top" then --update position
							print("Detected EarthShield ability changed.")
							KSSAnchor_Command("top".." "..KSSAnchor_AttachFrame)
						end
					end
				elseif KSSAnchor_Method ~= "off" and HealBot_Config.DisabledNow == 1 then
					--we are suppose to be anchored to healbot but it is disabled
					if relativeTo == FrameTable then --is it anchored to HealBot?
						--we are anchored to healbot but healbot is disabled so stop anchoring until it is healbot is enabled
						KSSAnchor_Command("disable")
					end
				end
			end
			LastUpdate = 0
		end
	end
	
end

function KSSAnchor_OnEvent(self,event,...)
	if event == "ADDON_LOADED" and select(1,...) == "KSSAnchor" then
		VarsLoaded = true
	end
end